# encoding: utf-8

require 'term/ansicolor'
# mixin String class for term-color methods
class String; include Term::ANSIColor end

require 'yaml'
require 'pp'
require 'hash_deep_merge'
require 'tempfile'
require 'diffy'
require 'tilt'
require 'ostruct'

require 'app_stack/version'
require 'app_stack/cli_options'
require 'app_stack/configuration'
require 'app_stack/app'
require 'app_stack/compare_list'
require 'app_stack/operator'

# module functions for handle stackup
module AppStack
  attr_reader :options

  DEFAULT_CONF_FILE = %w[app_stack.yml .app_stack.yml]

  # handle the execution to main class:
  def stackup!(argv = ARGV)
    @options = CliOptions.new(argv)
    conf_file = options.conf_file || find_conf_file('.')
    App.new(conf_file).stackup!
  end

  private

  # find configuration file (stackup.yml ...) in order
  # prefer the base name of conf-file used for stackup
  def find_conf_file(directory)
    conf_files = DEFAULT_CONF_FILE
    if options && options.conf_file
      base_name = File.basename(options.conf_file)
      conf_files.delete(base_name)
      conf_files.unshift(base_name)
    end

    conf_files.each do |f|
      return File.join(directory, f) if File.exists?(File.join(directory, f))
    end
    nil
  end

  module_function :stackup!, :options, :find_conf_file
end
