# encoding: utf-8

module AppStack
  # represent a single stacked application
  class App
    attr_accessor :rel_path
    attr_reader :config, :conf_file, :app_stacks, :export_groups,
                :sync_list, :import_list, :render_list,
                :compare_list, :copy_list, :diff_list, :attrs

    include AppStack::ConfigParser

    def initialize(filename = nil, app_dir = nil)
      @conf_file, @app_dir = filename, app_dir
      @config = load_config
      info 'loaded configuration: ', @config

      @app_stacks = {}
      @compare_list = []
      @attrs = {}
    end

    # rubocop:disable MethodLength
    def load_stack!
      # load_stack.each ...
      validate_config!

      # parse all stacks
      @import_list.merge(@sync_list).merge(@render_list).keys.each do |app|
        app_dir = File.expand_path(app, File.join(directory, config.stack_dir))
        # rubocop:disable LineLength
        fail ParseError, "Error on #{@conf_file}, stack app #{app} not found in #{app_dir}" unless File.directory?(app_dir)
        # rubocop:enable LineLength
        @app_stacks[app] = App.new(AppStack.find_conf_file(app_dir), app_dir)
        @app_stacks[app].rel_path = File.join(directory, config.stack_dir, app)
        @app_stacks[app].parse_export_groups!
        @attrs = @attrs.deep_merge @app_stacks[app].config.attrs

        # info "load app #{app.bold} as", @app_stacks[app]
        info "load app #{app.bold}"
      end

      @attrs = @attrs.deep_merge config.attrs
      info 'merged attrs', attrs
      @app_stacks
    end
    # rubocop:enable MethodLength

    def stackup!
      load_stack!
      load_compare_list!
      build_copy_list!
      options.reverse ? do_reverse! : merge!
    end

    def options
      AppStack.options
    end

    def directory
      @app_dir ||= (conf_file && File.dirname(conf_file))
      File.expand_path(@app_dir)
    end

    # use the direct name of the config file as app name
    def app_name
      directory && File.basename(directory)
    end

    # echo message only if verbose specified to be true
    # rubocop:disable CyclomaticComplexity
    def info(msg, var = nil)
      return unless options && options.verbose

      msg = "[#{app_name || self.class.name}] ".green + msg
      msg += "\n" if var && var.inspect.size > 30

      print msg
      if var
        print Term::ANSIColor.bold
        var.inspect.size > 30 ? PP.pp(var) : PP.singleline_pp(var)
        print Term::ANSIColor.reset
      end
      puts unless var && var.inspect.size > 30
    end
  end
end
