# encoding: utf-8

module AppStack
  # mixin app class for build copy list
  # rubocop:disable MethodLength
  class App
    def build_copy_list!
      @copy_list, @diff_list = [], []
      compare_list.each do |f|
        info f.label
        if File.exists?(f.to_file)
          if f.import?
            info f.skip('target exists')
            next
          end

          if f.diff?
            info f.label, 'add to diff list'
            @diff_list << f
          else
            info f.skip('not changed')
            next
          end
        end
        info f.label, 'add to copy list'
        @copy_list << f
      end
    end

    def merge!
      case
      when AppStack.options.print then do_print!
      when AppStack.options.force then do_copy!
      when AppStack.options.simulate then do_simulate!
      else diff_list.size > 0 ? do_warn : do_copy!
      end
    end

    def do_reverse!
      @rev_list = {}
      @diff_list.each_with_index do |f, i|
        f.reverse!
        puts "[#{i + 1}] " + f.reverse_label
        @rev_list[i + 1] = f
      end

      return unless @rev_list.keys.size > 0
      puts "Which file you want to copy reversely?"
      print "(1 2 ...):"
      lists = gets.chomp.split(/\s+/)
      lists.each { |i| fail "unknown option #{i}" unless @rev_list[i.to_i] }
      lists.each { |i| @rev_list[i.to_i].process }
    end

    def do_print!
      print_list = {}
      compare_list.each do |f|
        next unless File.exists?(f.to_file)

        print_list[f.to_file] = f
      end

      mapper = {}
      app_stacks.keys.each { |k| mapper[k] = [] }
      print_list.keys.sort.each do |k|
        f = print_list[k]
        mapper[f.from_app.app_name] << f
      end

      mapper.each do |k, list|
        puts "From #{k.to_s.bold.green}"
        list.each { |f| puts "  - #{f.to_file_short_name.blue}" }
      end
    end

    private

    def do_copy!
      @copy_list.map(&:process)
    end

    def do_simulate!
      @copy_list.each { |f| puts f.label }
    end

    def do_warn
      puts 'These files are changed remotely:'
      i = 1
      @diff_list.each do |f|
        puts "[#{i.to_s.bold}] " + f.to_file_short_name.blue
        puts "<- #{f.from_app.app_name.bold.blue} (#{f.from_file_short_name})"
        puts f.diff.to_s(:color)
        i += 1
      end
      puts 'use -f (--force) option to overwrite all files.'
    end
  end
end
